@@ -1,12 +1,14 @@
 /*
  * aconf-win32.h
  *
- * Copyright 2002 Glyph & Cog, LLC
+ * Copyright 2002-2003 Glyph & Cog, LLC
  */
 
 #ifndef ACONF_H
 #define ACONF_H
 
+#include <aconf2.h>
+
 /*
  * Use A4 paper size instead of Letter for PostScript output.
  */
@@ -23,9 +25,24 @@
 #undef OPI_SUPPORT
 
 /*
- * Use gzip instead of uncompress.
+ * Enable multithreading support.
  */
-#define USE_GZIP 1
+#define MULTITHREADED 1
+
+/*
+ * Enable C++ exceptions.
+ */
+#define USE_EXCEPTIONS 1
+
+/*
+ * Enable word list support.
+ */
+#undef TEXTOUT_WORD_LIST
+
+/*
+ * Use fixed point (instead of floating point) arithmetic.
+ */
+#undef USE_FIXEDPOINT
 
 /*
  * Directory with the Xpdf app-defaults file.
@@ -50,8 +67,14 @@
 #undef HAVE_BSTRING_H
 #undef HAVE_POPEN
 #undef HAVE_MKSTEMP
+#undef HAVE_MKSTEMPS
 #undef SELECT_TAKES_INT
+#undef HAVE_FSEEKO
 #undef HAVE_FSEEK64
+#undef _FILE_OFFSET_BITS
+#undef _LARGE_FILES
+#undef _LARGEFILE_SOURCE
+#undef HAVE_XTAPPSETEXITFLAG
 
 /*
  * This is defined if using libXpm.
@@ -67,16 +90,31 @@
  * One of these is defined if using FreeType (version 1 or 2).
  */
 #undef HAVE_FREETYPE_H
-#undef HAVE_FREETYPE_FREETYPE_H
+#define HAVE_FREETYPE_FREETYPE_H 1
 
 /*
  * This is defined if using FreeType version 2.
  */
-#undef FREETYPE2
+#define FREETYPE2
 
 /*
  * This is defined if using libpaper.
  */
 #undef HAVE_PAPER_H
 
+/*
+ * Enable support for loading plugins.
+ */
+#undef ENABLE_PLUGINS
+
+/*
+ * Defined if the Splash library is avaiable.
+ */
+#undef HAVE_SPLASH
+
+/*
+ * Enable support for CMYK output.
+ */
+#undef SPLASH_CMYK
+
 #endif
