@@ -23,6 +23,7 @@ class Array;
 class Stream;
 class Parser;
 class Dict;
+class Function;
 class OutputDev;
 class GfxFontDict;
 class GfxFont;
@@ -38,11 +39,11 @@ class GfxPatchMeshShading;
 struct GfxPatch;
 class GfxState;
 struct GfxColor;
+class GfxColorSpace;
 class Gfx;
 class PDFRectangle;
+class AnnotBorderStyle;
 
-//------------------------------------------------------------------------
-// Gfx
 //------------------------------------------------------------------------
 
 enum GfxClipType {
@@ -63,7 +64,7 @@ enum TchkType {
   tchkNone			// used to avoid empty initializer lists
 };
 
-#define maxArgs 8
+#define maxArgs 33
 
 struct Operator {
   char name[4];
@@ -72,6 +73,8 @@ struct Operator {
   void (Gfx::*func)(Object args[], int numArgs);
 };
 
+//------------------------------------------------------------------------
+
 class GfxResources {
 public:
 
@@ -99,6 +102,10 @@ private:
   GfxResources *next;
 };
 
+//------------------------------------------------------------------------
+// Gfx
+//------------------------------------------------------------------------
+
 class Gfx {
 public:
 
@@ -120,10 +127,10 @@ public:
   // Interpret a stream or array of streams.
   void display(Object *obj, GBool topLevel = gTrue);
 
-  // Display an annotation, given its appearance (a Form XObject) and
-  // bounding box (in default user space).
-  void doAnnot(Object *str, double xMin, double yMin,
-	       double xMax, double yMax);
+  // Display an annotation, given its appearance (a Form XObject),
+  // border style, and bounding box (in default user space).
+  void drawAnnot(Object *str, AnnotBorderStyle *borderStyle,
+		 double xMin, double yMin, double xMax, double yMax);
 
   // Save graphics state.
   void saveState();
@@ -176,6 +183,10 @@ private:
   void opSetMiterLimit(Object args[], int numArgs);
   void opSetLineWidth(Object args[], int numArgs);
   void opSetExtGState(Object args[], int numArgs);
+  void doSoftMask(Object *str, GBool alpha,
+		  GfxColorSpace *blendingColorSpace,
+		  GBool isolated, GBool knockout,
+		  Function *transferFunc, GfxColor *backdropColor);
   void opSetRenderingIntent(Object args[], int numArgs);
 
   // color operators
@@ -212,8 +223,11 @@ private:
   void opEOFillStroke(Object args[], int numArgs);
   void opCloseEOFillStroke(Object args[], int numArgs);
   void doPatternFill(GBool eoFill);
-  void doTilingPatternFill(GfxTilingPattern *tPat, GBool eoFill);
-  void doShadingPatternFill(GfxShadingPattern *sPat, GBool eoFill);
+  void doPatternStroke();
+  void doTilingPatternFill(GfxTilingPattern *tPat,
+			   GBool stroke, GBool eoFill);
+  void doShadingPatternFill(GfxShadingPattern *sPat,
+			    GBool stroke, GBool eoFill);
   void opShFill(Object args[], int numArgs);
   void doFunctionShFill(GfxFunctionShading *shading);
   void doFunctionShFill1(GfxFunctionShading *shading,
@@ -265,7 +279,12 @@ private:
   void opXObject(Object args[], int numArgs);
   void doImage(Object *ref, Stream *str, GBool inlineImg);
   void doForm(Object *str);
-  void doForm1(Object *str, Dict *resDict, double *matrix, double *bbox);
+  void doForm1(Object *str, Dict *resDict, double *matrix, double *bbox,
+	       GBool transpGroup = gFalse, GBool softMask = gFalse,
+	       GfxColorSpace *blendingColorSpace = NULL,
+	       GBool isolated = gFalse, GBool knockout = gFalse,
+	       GBool alpha = gFalse, Function *transferFunc = NULL,
+	       GfxColor *backdropColor = NULL);
 
   // in-line image operators
   void opBeginImage(Object args[], int numArgs);
