@@ -12,6 +12,7 @@
 #pragma implementation
 #endif
 
+#include <limits.h>
 #include "gmem.h"
 #include "Error.h"
 #include "JArithmeticDecoder.h"
@@ -176,6 +177,59 @@ static Guint signContext[5][5][2] = {
 
 //------------------------------------------------------------------------
 
+#if 1 //----- disable coverage tracking
+
+#define cover(idx)
+
+#else //----- enable coverage tracking
+
+class JPXCover {
+public:
+
+  JPXCover(int sizeA);
+  ~JPXCover();
+  void incr(int idx);
+
+private:
+
+  int size, used;
+  int *data;
+};
+
+JPXCover::JPXCover(int sizeA) {
+  size = sizeA;
+  used = -1;
+  data = (int *)gmallocn(size, sizeof(int));
+  memset(data, 0, size * sizeof(int));
+}
+
+JPXCover::~JPXCover() {
+  int i;
+
+  printf("JPX coverage:\n");
+  for (i = 0; i <= used; ++i) {
+    printf("  %4d: %8d\n", i, data[i]);
+  }
+  gfree(data);
+}
+
+void JPXCover::incr(int idx) {
+  if (idx < size) {
+    ++data[idx];
+    if (idx > used) {
+      used = idx;
+    }
+  }
+}
+
+JPXCover jpxCover(150);
+
+#define cover(idx) jpxCover.incr(idx)
+
+#endif //----- coverage tracking
+
+//------------------------------------------------------------------------
+
 JPXStream::JPXStream(Stream *strA):
   FilterStream(strA)
 {
@@ -195,6 +249,24 @@ JPXStream::JPXStream(Stream *strA):
 }
 
 JPXStream::~JPXStream() {
+  close();
+  delete str;
+}
+
+void JPXStream::reset() {
+  str->reset();
+  if (readBoxes()) {
+    curY = img.yOffset;
+  } else {
+    // readBoxes reported an error, so we go immediately to EOF
+    curY = img.ySize;
+  }
+  curX = img.xOffset;
+  curComp = 0;
+  readBufLen = 0;
+}
+
+void JPXStream::close() {
   JPXTile *tile;
   JPXTileComp *tileComp;
   JPXResLevel *resLevel;
@@ -204,19 +276,23 @@ JPXStream::~JPXStream() {
   Guint comp, i, k, r, pre, sb;
 
   gfree(bpc);
+  bpc = NULL;
   if (havePalette) {
     gfree(palette.bpc);
     gfree(palette.c);
+    havePalette = gFalse;
   }
   if (haveCompMap) {
     gfree(compMap.comp);
     gfree(compMap.type);
     gfree(compMap.pComp);
+    haveCompMap = gFalse;
   }
   if (haveChannelDefn) {
     gfree(channelDefn.idx);
     gfree(channelDefn.type);
     gfree(channelDefn.assoc);
+    haveChannelDefn = gFalse;
   }
 
   if (img.tiles) {
@@ -235,7 +311,7 @@ JPXStream::~JPXStream() {
 		for (pre = 0; pre < 1; ++pre) {
 		  precinct = &resLevel->precincts[pre];
 		  if (precinct->subbands) {
-		    for (sb = 0; sb < (r == 0 ? 1 : 3); ++sb) {
+		    for (sb = 0; sb < (Guint)(r == 0 ? 1 : 3); ++sb) {
 		      subband = &precinct->subbands[sb];
 		      gfree(subband->inclusion);
 		      gfree(subband->zeroBitPlane);
@@ -266,21 +342,9 @@ JPXStream::~JPXStream() {
       }
     }
     gfree(img.tiles);
+    img.tiles = NULL;
   }
-  delete str;
-}
-
-void JPXStream::reset() {
-  str->reset();
-  if (readBoxes()) {
-    curY = img.yOffset;
-  } else {
-    // readBoxes reported an error, so we go immediately to EOF
-    curY = img.ySize;
-  }
-  curX = img.xOffset;
-  curComp = 0;
-  readBufLen = 0;
+  FilterStream::close();
 }
 
 int JPXStream::getChar() {
@@ -393,8 +457,10 @@ void JPXStream::getImageParams(int *bitsPerComponent,
   } else {
     while (readBoxHdr(&boxType, &boxLen, &dataLen)) {
       if (boxType == 0x6a703268) { // JP2 header
+	cover(0);
 	// skip the superbox
       } else if (boxType == 0x69686472) { // image header
+	cover(1);
 	if (readULong(&dummy) &&
 	    readULong(&dummy) &&
 	    readUWord(&dummy) &&
@@ -406,6 +472,7 @@ void JPXStream::getImageParams(int *bitsPerComponent,
 	  haveBPC = gTrue;
 	}
       } else if (boxType == 0x636F6C72) { // color specification
+	cover(2);
 	if (readByte(&csMeth) &&
 	    readByte(&csPrec1) &&
 	    readByte(&dummy2)) {
@@ -439,11 +506,13 @@ void JPXStream::getImageParams(int *bitsPerComponent,
 	  }
 	}
       } else if (boxType == 0x6A703263) { // codestream
+	cover(3);
 	if (!(haveBPC && haveCSMode)) {
 	  getImageParams2(bitsPerComponent, csMode);
 	}
 	break;
       } else {
+	cover(4);
 	for (i = 0; i < dataLen; ++i) {
 	  str->getChar();
 	}
@@ -461,6 +530,7 @@ void JPXStream::getImageParams2(int *bitsPerComponent,
 
   while (readMarkerHdr(&segType, &segLen)) {
     if (segType == 0x51) { // SIZ - image and tile size
+      cover(5);
       if (readUWord(&dummy) &&
 	  readULong(&dummy) &&
 	  readULong(&dummy) &&
@@ -484,6 +554,7 @@ void JPXStream::getImageParams2(int *bitsPerComponent,
       }
       break;
     } else {
+      cover(6);
       if (segLen > 2) {
 	for (i = 0; i < segLen - 2; ++i) {
 	  str->getChar();
@@ -504,6 +575,7 @@ GBool JPXStream::readBoxes() {
   // wrapper) -- this appears to be a violation of the PDF spec, but
   // Acrobat allows it
   if (str->lookChar() == 0xff) {
+    cover(7);
     error(getPos(), "Naked JPEG 2000 codestream, missing JP2/JPX wrapper");
     readCodestream(0);
     nComps = img.nComps;
@@ -524,8 +596,10 @@ GBool JPXStream::readBoxes() {
       // some things which should be subboxes of the JP2 header box
       // show up outside of it - so we simply ignore the JP2 header
       // box
+      cover(8);
       break;
     case 0x69686472:		// image header
+      cover(9);
       if (!readULong(&height) ||
 	  !readULong(&width) ||
 	  !readUWord(&nComps) ||
@@ -547,6 +621,7 @@ GBool JPXStream::readBoxes() {
       haveImgHdr = gTrue;
       break;
     case 0x62706363:		// bits per component
+      cover(10);
       if (!haveImgHdr) {
 	error(getPos(), "Found bits per component box before image header box in JPX stream");
 	return gFalse;
@@ -563,11 +638,13 @@ GBool JPXStream::readBoxes() {
       }
       break;
     case 0x636F6C72:		// color specification
+      cover(11);
       if (!readColorSpecBox(dataLen)) {
 	return gFalse;
       }
       break;
     case 0x70636c72:		// palette
+      cover(12);
       if (!readUWord(&palette.nEntries) ||
 	  !readUByte(&palette.nComps)) {
 	error(getPos(), "Unexpected EOF in JPX stream");
@@ -596,6 +673,7 @@ GBool JPXStream::readBoxes() {
       havePalette = gTrue;
       break;
     case 0x636d6170:		// component mapping
+      cover(13);
       compMap.nChannels = dataLen / 4;
       compMap.comp = (Guint *)gmallocn(compMap.nChannels, sizeof(Guint));
       compMap.type = (Guint *)gmallocn(compMap.nChannels, sizeof(Guint));
@@ -611,6 +689,7 @@ GBool JPXStream::readBoxes() {
       haveCompMap = gTrue;
       break;
     case 0x63646566:		// channel definition
+      cover(14);
       if (!readUWord(&channelDefn.nChannels)) {
 	error(getPos(), "Unexpected EOF in JPX stream");
 	return gFalse;
@@ -632,6 +711,7 @@ GBool JPXStream::readBoxes() {
       haveChannelDefn = gTrue;
       break;
     case 0x6A703263:		// contiguous codestream
+      cover(15);
       if (!bpc) {
 	error(getPos(), "JPX stream is missing the image header box");
       }
@@ -643,6 +723,7 @@ GBool JPXStream::readBoxes() {
       }
       break;
     default:
+      cover(16);
       for (i = 0; i < dataLen; ++i) {
 	if (str->getChar() == EOF) {
 	  error(getPos(), "Unexpected EOF in JPX stream");
@@ -669,6 +750,7 @@ GBool JPXStream::readColorSpecBox(Guint dataLen) {
   }
   switch (newCS.meth) {
   case 1:			// enumerated colorspace
+    cover(17);
     if (!readULong(&csEnum)) {
       goto err;
     }
@@ -711,6 +793,7 @@ GBool JPXStream::readColorSpecBox(Guint dataLen) {
 	}
       } else if (dataLen == 7) {
 	//~ this assumes the 8-bit case
+	cover(92);
 	newCS.enumerated.cieLab.rl = 100;
 	newCS.enumerated.cieLab.ol = 0;
 	newCS.enumerated.cieLab.ra = 255;
@@ -757,6 +840,7 @@ GBool JPXStream::readColorSpecBox(Guint dataLen) {
   case 2:			// restricted ICC profile
   case 3: 			// any ICC profile (JPX)
   case 4:			// vendor color (JPX)
+    cover(18);
     for (i = 0; i < dataLen - 3; ++i) {
       if (str->getChar() == EOF) {
 	goto err;
@@ -795,8 +879,10 @@ GBool JPXStream::readCodestream(Guint len) {
     switch (segType) {
     case 0x4f:			// SOC - start of codestream
       // marker only
+      cover(19);
       break;
     case 0x51:			// SIZ - image and tile size
+      cover(20);
       if (!readUWord(&capabilities) ||
 	  !readULong(&img.xSize) ||
 	  !readULong(&img.ySize) ||
@@ -818,6 +904,12 @@ GBool JPXStream::readCodestream(Guint len) {
 	            / img.xTileSize;
       img.nYTiles = (img.ySize - img.yTileOffset + img.yTileSize - 1)
 	            / img.yTileSize;
+      // check for overflow before allocating memory
+      if (img.nXTiles <= 0 || img.nYTiles <= 0 ||
+	  img.nXTiles >= INT_MAX / img.nYTiles) {
+	error(getPos(), "Bad tile count in JPX SIZ marker segment");
+	return gFalse;
+      }
       img.tiles = (JPXTile *)gmallocn(img.nXTiles * img.nYTiles,
 				      sizeof(JPXTile));
       for (i = 0; i < img.nXTiles * img.nYTiles; ++i) {
@@ -848,6 +940,7 @@ GBool JPXStream::readCodestream(Guint len) {
       haveSIZ = gTrue;
       break;
     case 0x52:			// COD - coding style default
+      cover(21);
       if (!readUByte(&img.tiles[0].tileComps[0].style) ||
 	  !readUByte(&img.tiles[0].progOrder) ||
 	  !readUWord(&img.tiles[0].nLayers) ||
@@ -894,6 +987,7 @@ GBool JPXStream::readCodestream(Guint len) {
       }
       for (r = 0; r <= img.tiles[0].tileComps[0].nDecompLevels; ++r) {
 	if (img.tiles[0].tileComps[0].style & 0x01) {
+	  cover(91);
 	  if (!readUByte(&precinctSize)) {
 	    error(getPos(), "Error in JPX COD marker segment");
 	    return gFalse;
@@ -922,6 +1016,7 @@ GBool JPXStream::readCodestream(Guint len) {
       haveCOD = gTrue;
       break;
     case 0x53:			// COC - coding style component
+      cover(22);
       if (!haveCOD) {
 	error(getPos(), "JPX COC marker segment before COD segment");
 	return gFalse;
@@ -991,6 +1086,7 @@ GBool JPXStream::readCodestream(Guint len) {
       }
       break;
     case 0x5c:			// QCD - quantization default
+      cover(23);
       if (!readUByte(&img.tiles[0].tileComps[0].quantStyle)) {
 	error(getPos(), "Error in JPX QCD marker segment");
 	return gFalse;
@@ -1054,6 +1150,7 @@ GBool JPXStream::readCodestream(Guint len) {
       haveQCD = gTrue;
       break;
     case 0x5d:			// QCC - quantization component
+      cover(24);
       if (!haveQCD) {
 	error(getPos(), "JPX QCC marker segment before QCD segment");
 	return gFalse;
@@ -1121,6 +1218,7 @@ GBool JPXStream::readCodestream(Guint len) {
       }
       break;
     case 0x5e:			// RGN - region of interest
+      cover(25);
 #if 1 //~ ROI is unimplemented
       fprintf(stderr, "RGN\n");
       for (i = 0; i < segLen - 2; ++i) {
@@ -1141,6 +1239,7 @@ GBool JPXStream::readCodestream(Guint len) {
 #endif
       break;
     case 0x5f:			// POC - progression order change
+      cover(26);
 #if 1 //~ progression order changes are unimplemented
       fprintf(stderr, "POC\n");
       for (i = 0; i < segLen - 2; ++i) {
@@ -1168,6 +1267,7 @@ GBool JPXStream::readCodestream(Guint len) {
 #endif
       break;
     case 0x60:			// PPM - packed packet headers, main header
+      cover(27);
 #if 1 //~ packed packet headers are unimplemented
       fprintf(stderr, "PPM\n");
       for (i = 0; i < segLen - 2; ++i) {
@@ -1180,6 +1280,7 @@ GBool JPXStream::readCodestream(Guint len) {
       break;
     case 0x55:			// TLM - tile-part lengths
       // skipped
+      cover(28);
       for (i = 0; i < segLen - 2; ++i) {
 	if (str->getChar() == EOF) {
 	  error(getPos(), "Error in JPX TLM marker segment");
@@ -1189,6 +1290,7 @@ GBool JPXStream::readCodestream(Guint len) {
       break;
     case 0x57:			// PLM - packet length, main header
       // skipped
+      cover(29);
       for (i = 0; i < segLen - 2; ++i) {
 	if (str->getChar() == EOF) {
 	  error(getPos(), "Error in JPX PLM marker segment");
@@ -1198,6 +1300,7 @@ GBool JPXStream::readCodestream(Guint len) {
       break;
     case 0x63:			// CRG - component registration
       // skipped
+      cover(30);
       for (i = 0; i < segLen - 2; ++i) {
 	if (str->getChar() == EOF) {
 	  error(getPos(), "Error in JPX CRG marker segment");
@@ -1207,6 +1310,7 @@ GBool JPXStream::readCodestream(Guint len) {
       break;
     case 0x64:			// COM - comment
       // skipped
+      cover(31);
       for (i = 0; i < segLen - 2; ++i) {
 	if (str->getChar() == EOF) {
 	  error(getPos(), "Error in JPX COM marker segment");
@@ -1215,9 +1319,11 @@ GBool JPXStream::readCodestream(Guint len) {
       }
       break;
     case 0x90:			// SOT - start of tile
+      cover(32);
       haveSOT = gTrue;
       break;
     default:
+      cover(33);
       error(getPos(), "Unknown marker segment %02x in JPX stream", segType);
       for (i = 0; i < segLen - 2; ++i) {
 	if (str->getChar() == EOF) {
@@ -1318,6 +1424,7 @@ GBool JPXStream::readTilePart() {
     tilePartLen -= 2 + segLen;
     switch (segType) {
     case 0x52:			// COD - coding style default
+      cover(34);
       if (!readUByte(&img.tiles[tileIdx].tileComps[0].style) ||
 	  !readUByte(&img.tiles[tileIdx].progOrder) ||
 	  !readUWord(&img.tiles[tileIdx].nLayers) ||
@@ -1385,6 +1492,7 @@ GBool JPXStream::readTilePart() {
       }
       break;
     case 0x53:			// COC - coding style component
+      cover(35);
       if ((img.nComps > 256 && !readUWord(&comp)) ||
 	  (img.nComps <= 256 && !readUByte(&comp)) ||
 	  comp >= img.nComps ||
@@ -1426,6 +1534,7 @@ GBool JPXStream::readTilePart() {
       }
       break;
     case 0x5c:			// QCD - quantization default
+      cover(36);
       if (!readUByte(&img.tiles[tileIdx].tileComps[0].quantStyle)) {
 	error(getPos(), "Error in JPX QCD marker segment");
 	return gFalse;
@@ -1485,6 +1594,7 @@ GBool JPXStream::readTilePart() {
       }
       break;
     case 0x5d:			// QCC - quantization component
+      cover(37);
       if ((img.nComps > 256 && !readUWord(&comp)) ||
 	  (img.nComps <= 256 && !readUByte(&comp)) ||
 	  comp >= img.nComps ||
@@ -1536,6 +1646,7 @@ GBool JPXStream::readTilePart() {
       }
       break;
     case 0x5e:			// RGN - region of interest
+      cover(38);
 #if 1 //~ ROI is unimplemented
       fprintf(stderr, "RGN\n");
       for (i = 0; i < segLen - 2; ++i) {
@@ -1556,6 +1667,7 @@ GBool JPXStream::readTilePart() {
 #endif
       break;
     case 0x5f:			// POC - progression order change
+      cover(39);
 #if 1 //~ progression order changes are unimplemented
       fprintf(stderr, "POC\n");
       for (i = 0; i < segLen - 2; ++i) {
@@ -1583,6 +1695,7 @@ GBool JPXStream::readTilePart() {
 #endif
       break;
     case 0x61:			// PPT - packed packet headers, tile-part hdr
+      cover(40);
 #if 1 //~ packed packet headers are unimplemented
       fprintf(stderr, "PPT\n");
       for (i = 0; i < segLen - 2; ++i) {
@@ -1594,6 +1707,7 @@ GBool JPXStream::readTilePart() {
 #endif
     case 0x58:			// PLT - packet length, tile-part header
       // skipped
+      cover(41);
       for (i = 0; i < segLen - 2; ++i) {
 	if (str->getChar() == EOF) {
 	  error(getPos(), "Error in JPX PLT marker segment");
@@ -1603,6 +1717,7 @@ GBool JPXStream::readTilePart() {
       break;
     case 0x64:			// COM - comment
       // skipped
+      cover(42);
       for (i = 0; i < segLen - 2; ++i) {
 	if (str->getChar() == EOF) {
 	  error(getPos(), "Error in JPX COM marker segment");
@@ -1611,9 +1726,11 @@ GBool JPXStream::readTilePart() {
       }
       break;
     case 0x93:			// SOD - start of data
+      cover(43);
       haveSOD = gTrue;
       break;
     default:
+      cover(44);
       error(getPos(), "Unknown marker segment %02x in JPX tile-part stream",
 	    segType);
       for (i = 0; i < segLen - 2; ++i) {
@@ -1811,6 +1928,7 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
   while (1) {
     if (tilePartToEOC) {
       //~ peek for an EOC marker
+      cover(93);
     } else if (tilePartLen == 0) {
       break;
     }
@@ -1821,13 +1939,17 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
 
     //----- packet header
 
+    // setup
+    startBitBuf(tilePartLen);
+
     // zero-length flag
     if (!readBits(1, &bits)) {
       goto err;
     }
     if (!bits) {
       // packet is empty -- clear all code-block inclusion flags
-      for (sb = 0; sb < (tile->res == 0 ? 1 : 3); ++sb) {
+      cover(45);
+      for (sb = 0; sb < (Guint)(tile->res == 0 ? 1 : 3); ++sb) {
 	subband = &precinct->subbands[sb];
 	for (cbY = 0; cbY < subband->nYCBs; ++cbY) {
 	  for (cbX = 0; cbX < subband->nXCBs; ++cbX) {
@@ -1838,7 +1960,7 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
       }
     } else {
 
-      for (sb = 0; sb < (tile->res == 0 ? 1 : 3); ++sb) {
+      for (sb = 0; sb < (Guint)(tile->res == 0 ? 1 : 3); ++sb) {
 	subband = &precinct->subbands[sb];
 	for (cbY = 0; cbY < subband->nYCBs; ++cbY) {
 	  for (cbX = 0; cbX < subband->nXCBs; ++cbX) {
@@ -1846,16 +1968,19 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
 
 	    // skip code-blocks with no coefficients
 	    if (cb->x0 >= cb->x1 || cb->y0 >= cb->y1) {
+	      cover(46);
 	      cb->included = gFalse;
 	      continue;
 	    }
 
 	    // code-block inclusion
 	    if (cb->seen) {
+	      cover(47);
 	      if (!readBits(1, &cb->included)) {
 		goto err;
 	      }
 	    } else {
+	      cover(48);
 	      ttVal = 0;
 	      i = 0;
 	      for (level = subband->maxTTLevel; level >= 0; --level) {
@@ -1889,9 +2014,11 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
 	    }
 
 	    if (cb->included) {
+	      cover(49);
 
 	      // zero bit-plane count
 	      if (!cb->seen) {
+		cover(50);
 		ttVal = 0;
 		i = 0;
 		for (level = subband->maxTTLevel; level >= 0; --level) {
@@ -1925,26 +2052,33 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
 		goto err;
 	      }
 	      if (bits == 0) {
+		cover(51);
 		cb->nCodingPasses = 1;
 	      } else {
 		if (!readBits(1, &bits)) {
 		  goto err;
 		}
 		if (bits == 0) {
+		  cover(52);
 		  cb->nCodingPasses = 2;
 		} else {
+		  cover(53);
 		  if (!readBits(2, &bits)) {
 		    goto err;
 		  }
 		  if (bits < 3) {
+		    cover(54);
 		    cb->nCodingPasses = 3 + bits;
 		  } else {
+		    cover(55);
 		    if (!readBits(5, &bits)) {
 		      goto err;
 		    }
 		    if (bits < 31) {
+		      cover(56);
 		      cb->nCodingPasses = 6 + bits;
 		    } else {
+		      cover(57);
 		      if (!readBits(7, &bits)) {
 			goto err;
 		      }
@@ -1978,12 +2112,11 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
 	}
       }
     }
-    tilePartLen -= byteCount;
-    clearBitBuf();
+    tilePartLen = finishBitBuf();
 
     //----- packet data
 
-    for (sb = 0; sb < (tile->res == 0 ? 1 : 3); ++sb) {
+    for (sb = 0; sb < (Guint)(tile->res == 0 ? 1 : 3); ++sb) {
       subband = &precinct->subbands[sb];
       for (cbY = 0; cbY < subband->nYCBs; ++cbY) {
 	for (cbX = 0; cbX < subband->nXCBs; ++cbX) {
@@ -2004,6 +2137,7 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
 
     switch (tile->progOrder) {
     case 0: // layer, resolution level, component, precinct
+      cover(58);
       if (++tile->comp == img.nComps) {
 	tile->comp = 0;
 	if (++tile->res == tile->maxNDecompLevels + 1) {
@@ -2015,6 +2149,7 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
       }
       break;
     case 1: // resolution level, layer, component, precinct
+      cover(59);
       if (++tile->comp == img.nComps) {
 	tile->comp = 0;
 	if (++tile->layer == tile->nLayers) {
@@ -2027,6 +2162,7 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
       break;
     case 2: // resolution level, precinct, component, layer
       //~ this isn't correct -- see B.12.1.3
+      cover(60);
       if (++tile->layer == tile->nLayers) {
 	tile->layer = 0;
 	if (++tile->comp == img.nComps) {
@@ -2039,6 +2175,7 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
       break;
     case 3: // precinct, component, resolution level, layer
       //~ this isn't correct -- see B.12.1.4
+      cover(61);
       if (++tile->layer == tile->nLayers) {
 	tile->layer = 0;
 	if (++tile->res == tile->maxNDecompLevels + 1) {
@@ -2051,6 +2188,7 @@ GBool JPXStream::readTilePartData(Guint tileIdx,
       break;
     case 4: // component, precinct, resolution level, layer
       //~ this isn't correct -- see B.12.1.5
+      cover(62);
       if (++tile->layer == tile->nLayers) {
 	tile->layer = 0;
 	if (++tile->res == tile->maxNDecompLevels + 1) {
@@ -2083,8 +2221,10 @@ GBool JPXStream::readCodeBlockData(JPXTileComp *tileComp,
   Guint i, x, y0, y1, y2;
 
   if (cb->arithDecoder) {
+    cover(63);
     cb->arithDecoder->restart(cb->dataLen);
   } else {
+    cover(64);
     cb->arithDecoder = new JArithmeticDecoder();
     cb->arithDecoder->setStream(str, cb->dataLen);
     cb->arithDecoder->start();
@@ -2099,6 +2239,7 @@ GBool JPXStream::readCodeBlockData(JPXTileComp *tileComp,
 
     //----- significance propagation pass
     case jpxPassSigProp:
+      cover(65);
       for (y0 = cb->y0, coeff0 = cb->coeffs;
 	   y0 < cb->y1;
 	   y0 += 4, coeff0 += 4 << tileComp->codeBlockW) {
@@ -2176,6 +2317,7 @@ GBool JPXStream::readCodeBlockData(JPXTileComp *tileComp,
 
     //----- magnitude refinement pass
     case jpxPassMagRef:
+      cover(66);
       for (y0 = cb->y0, coeff0 = cb->coeffs;
 	   y0 < cb->y1;
 	   y0 += 4, coeff0 += 4 << tileComp->codeBlockW) {
@@ -2237,6 +2379,7 @@ GBool JPXStream::readCodeBlockData(JPXTileComp *tileComp,
 
     //----- cleanup pass
     case jpxPassCleanup:
+      cover(67);
       for (y0 = cb->y0, coeff0 = cb->coeffs;
 	   y0 < cb->y1;
 	   y0 += 4, coeff0 += 4 << tileComp->codeBlockW) {
@@ -2398,6 +2541,8 @@ void JPXStream::inverseTransform(JPXTileComp *tileComp) {
   Guint nx0, ny0, nx1, ny1;
   Guint r, cbX, cbY, x, y;
 
+  cover(68);
+
   //----- (NL)LL subband (resolution level 0)
 
   resLevel = &tileComp->resLevels[0];
@@ -2408,14 +2553,17 @@ void JPXStream::inverseTransform(JPXTileComp *tileComp) {
   qStyle = tileComp->quantStyle & 0x1f;
   guard = (tileComp->quantStyle >> 5) & 7;
   if (qStyle == 0) {
+    cover(69);
     eps = (tileComp->quantSteps[0] >> 3) & 0x1f;
     shift = guard + eps - 1;
     mu = 0; // make gcc happy
   } else {
+    cover(70);
     shift = guard - 1 + tileComp->prec;
     mu = (double)(0x800 + (tileComp->quantSteps[0] & 0x7ff)) / 2048.0;
   }
   if (tileComp->transform == 0) {
+    cover(71);
     shift += fracBits;
   }
 
@@ -2435,18 +2583,24 @@ void JPXStream::inverseTransform(JPXTileComp *tileComp) {
 	  if (val != 0) {
 	    shift2 = shift - (cb->nZeroBitPlanes + coeff->len);
 	    if (shift2 > 0) {
+	      cover(94);
 	      val = (val << shift2) + (1 << (shift2 - 1));
 	    } else {
+	      cover(95);
 	      val >>= -shift2;
 	    }
 	    if (qStyle == 0) {
+	      cover(96);
 	      if (tileComp->transform == 0) {
+		cover(97);
 		val &= -1 << fracBits;
 	      }
 	    } else {
+	      cover(98);
 	      val = (int)((double)val * mu);
 	    }
 	    if (coeff->flags & jpxCoeffSign) {
+	      cover(99);
 	      val = -val;
 	    }
 	  }
@@ -2467,11 +2621,13 @@ void JPXStream::inverseTransform(JPXTileComp *tileComp) {
     // and inverse transform to get (n-1)LL, which will be stored
     // in the upper-left corner of the tile-component data array
     if (r == tileComp->nDecompLevels) {
+      cover(72);
       nx0 = tileComp->x0;
       ny0 = tileComp->y0;
       nx1 = tileComp->x1;
       ny1 = tileComp->y1;
     } else {
+      cover(73);
       nx0 = tileComp->resLevels[r+1].x0;
       ny0 = tileComp->resLevels[r+1].y0;
       nx1 = tileComp->resLevels[r+1].x1;
@@ -2524,18 +2680,22 @@ void JPXStream::inverseTransformLevel(JPXTileComp *tileComp,
 
     // i-quant parameters
     if (qStyle == 0) {
+      cover(100);
       eps = (tileComp->quantSteps[3*r - 2 + sb] >> 3) & 0x1f;
       shift = guard + eps - 1;
       mu = 0; // make gcc happy
     } else {
+      cover(101);
       shift = guard + tileComp->prec;
       if (sb == 2) {
+	cover(102);
 	++shift;
       }
       t = tileComp->quantSteps[qStyle == 1 ? 0 : (3*r - 2 + sb)];
       mu = (double)(0x800 + (t & 0x7ff)) / 2048.0;
     }
     if (tileComp->transform == 0) {
+      cover(103);
       shift += fracBits;
     }
 
@@ -2558,18 +2718,23 @@ void JPXStream::inverseTransformLevel(JPXTileComp *tileComp,
 	    if (val != 0) {
 	      shift2 = shift - (cb->nZeroBitPlanes + coeff->len);
 	      if (shift2 > 0) {
+		cover(74);
 		val = (val << shift2) + (1 << (shift2 - 1));
 	      } else {
+		cover(75);
 		val >>= -shift2;
 	      }
 	      if (qStyle == 0) {
+		cover(76);
 		if (tileComp->transform == 0) {
 		  val &= -1 << fracBits;
 		}
 	      } else {
+		cover(77);
 		val = (int)((double)val * mu);
 	      }
 	      if (coeff->flags & jpxCoeffSign) {
+		cover(78);
 		val = -val;
 	      }
 	    }
@@ -2606,11 +2771,14 @@ void JPXStream::inverseTransform1D(JPXTileComp *tileComp,
 
   //----- special case for length = 1
   if (i1 - i0 == 1) {
+    cover(79);
     if (i0 & 1) {
+      cover(104);
       *data >>= 1;
     }
 
   } else {
+    cover(80);
 
     // choose an offset: this makes even buf[] indexes correspond to
     // odd values of i, and vice versa
@@ -2626,19 +2794,25 @@ void JPXStream::inverseTransform1D(JPXTileComp *tileComp,
     //----- extend right
     buf[end] = buf[end - 2];
     if (i1 - i0 == 2) {
+      cover(81);
       buf[end+1] = buf[offset + 1];
       buf[end+2] = buf[offset];
       buf[end+3] = buf[offset + 1];
     } else {
+      cover(82);
       buf[end+1] = buf[end - 3];
       if (i1 - i0 == 3) {
+	cover(105);
 	buf[end+2] = buf[offset + 1];
 	buf[end+3] = buf[offset + 2];
       } else {
+	cover(106);
 	buf[end+2] = buf[end - 4];
 	if (i1 - i0 == 4) {
+	  cover(107);
 	  buf[end+3] = buf[offset + 1];
 	} else {
+	  cover(108);
 	  buf[end+3] = buf[end - 5];
 	}
       }
@@ -2649,12 +2823,14 @@ void JPXStream::inverseTransform1D(JPXTileComp *tileComp,
     buf[offset - 2] = buf[offset + 2];
     buf[offset - 3] = buf[offset + 3];
     if (offset == 4) {
+      cover(83);
       buf[0] = buf[offset + 4];
     }
 
     //----- 9-7 irreversible filter
 
     if (tileComp->transform == 0) {
+      cover(84);
       // step 1 (even)
       for (i = 1; i <= end + 2; i += 2) {
 	buf[i] = (int)(idwtKappa * buf[i]);
@@ -2683,6 +2859,7 @@ void JPXStream::inverseTransform1D(JPXTileComp *tileComp,
     //----- 5-3 reversible filter
 
     } else {
+      cover(85);
       // step 1 (even)
       for (i = 3; i <= end; i += 2) {
 	buf[i] -= (buf[i-1] + buf[i+1] + 2) >> 2;
@@ -2711,6 +2888,7 @@ GBool JPXStream::inverseMultiCompAndDC(JPXTile *tile) {
   //----- inverse multi-component transform
 
   if (tile->multiComp == 1) {
+    cover(86);
     if (img.nComps < 3 ||
 	tile->tileComps[0].hSep != tile->tileComps[1].hSep ||
 	tile->tileComps[0].vSep != tile->tileComps[1].vSep ||
@@ -2721,6 +2899,7 @@ GBool JPXStream::inverseMultiCompAndDC(JPXTile *tile) {
 
     // inverse irreversible multiple component transform
     if (tile->tileComps[0].transform == 0) {
+      cover(87);
       j = 0;
       for (y = 0; y < tile->tileComps[0].y1 - tile->tileComps[0].y0; ++y) {
 	for (x = 0; x < tile->tileComps[0].x1 - tile->tileComps[0].x0; ++x) {
@@ -2737,6 +2916,7 @@ GBool JPXStream::inverseMultiCompAndDC(JPXTile *tile) {
 
     // inverse reversible multiple component transform
     } else {
+      cover(88);
       j = 0;
       for (y = 0; y < tile->tileComps[0].y1 - tile->tileComps[0].y0; ++y) {
 	for (x = 0; x < tile->tileComps[0].x1 - tile->tileComps[0].x0; ++x) {
@@ -2758,6 +2938,7 @@ GBool JPXStream::inverseMultiCompAndDC(JPXTile *tile) {
 
     // signed: clip
     if (tileComp->sgned) {
+      cover(89);
       minVal = -(1 << (tileComp->prec - 1));
       maxVal = (1 << (tileComp->prec - 1)) - 1;
       dataPtr = tileComp->data;
@@ -2765,11 +2946,14 @@ GBool JPXStream::inverseMultiCompAndDC(JPXTile *tile) {
 	for (x = 0; x < tileComp->x1 - tileComp->x0; ++x) {
 	  coeff = *dataPtr;
 	  if (tileComp->transform == 0) {
+	    cover(109);
 	    coeff >>= fracBits;
 	  }
 	  if (coeff < minVal) {
+	    cover(110);
 	    coeff = minVal;
 	  } else if (coeff > maxVal) {
+	    cover(111);
 	    coeff = maxVal;
 	  }
 	  *dataPtr++ = coeff;
@@ -2778,6 +2962,7 @@ GBool JPXStream::inverseMultiCompAndDC(JPXTile *tile) {
 
     // unsigned: inverse DC level shift and clip
     } else {
+      cover(90);
       maxVal = (1 << tileComp->prec) - 1;
       zeroVal = 1 << (tileComp->prec - 1);
       dataPtr = tileComp->data;
@@ -2785,12 +2970,15 @@ GBool JPXStream::inverseMultiCompAndDC(JPXTile *tile) {
 	for (x = 0; x < tileComp->x1 - tileComp->x0; ++x) {
 	  coeff = *dataPtr;
 	  if (tileComp->transform == 0) {
+	    cover(112);
 	    coeff >>= fracBits;
 	  }
 	  coeff += zeroVal;
 	  if (coeff < 0) {
+	    cover(113);
 	    coeff = 0;
 	  } else if (coeff > maxVal) {
+	    cover(114);
 	    coeff = maxVal;
 	  }
 	  *dataPtr++ = coeff;
@@ -2923,10 +3111,10 @@ GBool JPXStream::readBits(int nBits, Guint *x) {
   int c;
 
   while (bitBufLen < nBits) {
-    if ((c = str->getChar()) == EOF) {
+    if (byteCount == 0 || (c = str->getChar()) == EOF) {
       return gFalse;
     }
-    ++byteCount;
+    --byteCount;
     if (bitBufSkip) {
       bitBuf = (bitBuf << 7) | (c & 0x7f);
       bitBufLen += 7;
@@ -2941,8 +3129,16 @@ GBool JPXStream::readBits(int nBits, Guint *x) {
   return gTrue;
 }
 
-void JPXStream::clearBitBuf() {
+void JPXStream::startBitBuf(Guint byteCountA) {
   bitBufLen = 0;
   bitBufSkip = gFalse;
-  byteCount = 0;
+  byteCount = byteCountA;
+}
+
+Guint JPXStream::finishBitBuf() {
+  if (bitBufSkip) {
+    str->getChar();
+    --byteCount;
+  }
+  return byteCount;
 }
