@@ -20,14 +20,18 @@
 
 class GString;
 class GfxState;
+struct GfxColor;
 class GfxColorSpace;
 class GfxImageColorMap;
 class GfxFunctionShading;
 class GfxAxialShading;
 class GfxRadialShading;
 class Stream;
+class Links;
 class Link;
 class Catalog;
+class Page;
+class Function;
 
 //------------------------------------------------------------------------
 // OutputDev
@@ -61,6 +65,10 @@ public:
   // will be reduced to a series of other drawing operations.
   virtual GBool useShadedFills() { return gFalse; }
 
+  // Does this device use drawForm()?  If this returns false,
+  // form-type XObjects will be interpreted (i.e., unrolled).
+  virtual GBool useDrawForm() { return gFalse; }
+
   // Does this device use beginType3Char/endType3Char?  Otherwise,
   // text in Type 3 fonts will be drawn with drawChar/drawString.
   virtual GBool interpretType3Chars() = 0;
@@ -73,6 +81,18 @@ public:
   // Set default transform matrix.
   virtual void setDefaultCTM(double *ctm);
 
+  // Check to see if a page slice should be displayed.  If this
+  // returns false, the page display is aborted.  Typically, an
+  // OutputDev will use some alternate means to display the page
+  // before returning false.
+  virtual GBool checkPageSlice(Page *page, double hDPI, double vDPI,
+			       int rotate, GBool useMediaBox, GBool crop,
+			       int sliceX, int sliceY, int sliceW, int sliceH,
+			       GBool printing, Catalog *catalog,
+			       GBool (*abortCheckCbk)(void *data) = NULL,
+			       void *abortCheckCbkData = NULL)
+    { return gTrue; }
+
   // Start a page.
   virtual void startPage(int pageNum, GfxState *state) {}
 
@@ -91,9 +111,6 @@ public:
   double *getDefCTM() { return defCTM; }
   double *getDefICTM() { return defICTM; }
 
-  //----- link borders
-  virtual void drawLink(Link *link, Catalog *catalog) {}
-
   //----- save/restore graphics state
   virtual void saveState(GfxState *state) {}
   virtual void restoreState(GfxState *state) {}
@@ -108,6 +125,7 @@ public:
   virtual void updateLineCap(GfxState *state) {}
   virtual void updateMiterLimit(GfxState *state) {}
   virtual void updateLineWidth(GfxState *state) {}
+  virtual void updateStrokeAdjust(GfxState *state) {}
   virtual void updateFillColorSpace(GfxState *state) {}
   virtual void updateStrokeColorSpace(GfxState *state) {}
   virtual void updateFillColor(GfxState *state) {}
@@ -117,6 +135,7 @@ public:
   virtual void updateStrokeOpacity(GfxState *state) {}
   virtual void updateFillOverprint(GfxState *state) {}
   virtual void updateStrokeOverprint(GfxState *state) {}
+  virtual void updateTransfer(GfxState *state) {}
 
   //----- update text state
   virtual void updateFont(GfxState *state) {}
@@ -138,14 +157,18 @@ public:
 				 double *mat, double *bbox,
 				 int x0, int y0, int x1, int y1,
 				 double xStep, double yStep) {}
-  virtual void functionShadedFill(GfxState *state,
-				  GfxFunctionShading *shading) {}
-  virtual void axialShadedFill(GfxState *state, GfxAxialShading *shading) {}
-  virtual void radialShadedFill(GfxState *state, GfxRadialShading *shading) {}
+  virtual GBool functionShadedFill(GfxState *state,
+				   GfxFunctionShading *shading)
+    { return gFalse; }
+  virtual GBool axialShadedFill(GfxState *state, GfxAxialShading *shading)
+    { return gFalse; }
+  virtual GBool radialShadedFill(GfxState *state, GfxRadialShading *shading)
+    { return gFalse; }
 
   //----- path clipping
   virtual void clip(GfxState *state) {}
   virtual void eoClip(GfxState *state) {}
+  virtual void clipToStrokePath(GfxState *state) {}
 
   //----- text drawing
   virtual void beginStringOp(GfxState *state) {}
@@ -193,9 +216,31 @@ public:
   virtual void type3D1(GfxState *state, double wx, double wy,
 		       double llx, double lly, double urx, double ury) {}
 
+  //----- form XObjects
+  virtual void drawForm(Ref id) {}
+
   //----- PostScript XObjects
   virtual void psXObject(Stream *psStream, Stream *level1Stream) {}
 
+  //----- transparency groups and soft masks
+  virtual void beginTransparencyGroup(GfxState *state, double *bbox,
+				      GfxColorSpace *blendingColorSpace,
+				      GBool isolated, GBool knockout,
+				      GBool forSoftMask) {}
+  virtual void endTransparencyGroup(GfxState *state) {}
+  virtual void paintTransparencyGroup(GfxState *state, double *bbox) {}
+  virtual void setSoftMask(GfxState *state, double *bbox, GBool alpha,
+			   Function *transferFunc, GfxColor *backdropColor) {}
+  virtual void clearSoftMask(GfxState *state) {}
+
+  //----- links
+  virtual void processLink(Link *link, Catalog *catalog) {}
+
+#if 1 //~tmp: turn off anti-aliasing temporarily
+  virtual GBool getVectorAntialias() { return gFalse; }
+  virtual void setVectorAntialias(GBool vaa) {}
+#endif
+
 private:
 
   double defCTM[6];		// default coordinate transform matrix
